/* *****************************************************************
   Directory: /home/mchernoff/P219C_Mito/sas_programs
   Pgm: find_date_q1_meth.sas
   Source Pgm: find_date_q1.sas (Sep 29, 2006)
   Date: July 8, 2011
   Last edited: 7-11-11 - Reworked age at event since event dates
                          are now relative to birthdate. Added labels
                          for cond1-cond24 indicators.
                7-13-11 - Added notation.
   Purpose: 
            * MWG iDACS 616 - Case identification, Query 1

            * Prepared for publication

            * Note- All visit dates are expressed in days since
              participants birth. Input dataset has been de-indentified
              and patient identifiers have been blinded (randomly assigned).

            * Find date when patient first meets case definitions.

               * sortdt is the date that criterion is first met
                 (major or 2nd event of 2nd minor pair).
               * fmindate is the earliest date of scorable events,
                 including the major events and the
                 the first event of each of the qualifying minor 
                 pairs (mindate). Minage is based on fmindate.
               * sortdt and fmindate will only differ if there
                 were both major and minor conditions that
                 qualified on the same date.
               * See notes.
     
               * major/minor indicate which criterion was
                 met on earliest date
               * evmajor/evminor indicate if criteria were ever met.

             * minor conditions:  indx9-indx13 indx22-indx24
             * major conditions:  cond1-cond8 cond14-cond21


   Input files: mitometh.master
                mitometh.filtq1    - filtered gr2


   Output files: 

                 mitometh.date_fq1 - filtered, first scorable event meeting criterion (gr2)
                 mitometh.datall_fq1 - all records for scorable events meeting major or minor criterion
                                       this dataset will include a record for each event date that the
                                       major or minor condition is satisfied.


   Notes:   * dateseq holds the 1st date of the earliest abn sequences,
              if relevant, otherwise the sortdt. In some cases,
              dateseq was just assigned for first record of patid-condition
              (and not for subseqent records). This was done to retain
              uniqueness of event-records and sortdts.


            * Note- dxdat (all scorable conditions) is now saved [as of 7-2-07].
              Firstdx (1st record w/ a scorable condition) is already saved.

               ********************************************************************
               ** Note: for this dset (dxdat=datall_fq1, etc.,) the minor condition
                  indicators are carried to subsequent records once criterion is
                  met. For major conditions, the indicator is only 1 on the
                  visits that the condition occurred.
               ********************************************************************

*********************************************************************/

* automatic if in same directory *;
*%include "/home/mchernoff/P219C_Mito/methods/autoexec.sas";

options ls=130 ps=50;
run;


/**************************************************/
/* formats */
/**************************************************/

%include "/home/mchernoff/P219C_Mito/methods/mito_tmpformat_meth.sas"; * 7-11-11 *;

*------------------------------------------*;
* label macro *;
*------------------------------------------*;
%macro labmac (
   max= /*  number of conditions */
   );
 %do n=1 %to &max.;
   cond&n.="Condition &n."
 %end;
%mend;


*------------------------------------------*
* Bring in DATA    *
*------------------------------------------*;

x "gunzip /home/mchernoff/P219C_Mito/methods/master.sas7bdat.gz";

proc sort data=mitometh.master (keep=patid /*birthdt*/) out=master; * 7-11-11 *;
by patid;
run;

x "gzip /home/mchernoff/P219C_Mito/methods/master.sas7bdat";

*------------------------------------*;
* Apply scoring algorithm *;
*------------------------------------*;

* It is important to note that for the dataset which retains all the visit records for
  the first and subsequent qualifying record for the minor conditions-
  the indicators are retained and remain with value of 1 after the qualification
  criterion has been met - they cannot be used as indicators for the # times the
  minor condition occurred, just that it did occur. *;


x "gunzip /home/mchernoff/P219C_Mito/methods/filtq1.sas7bdat.gz";

* Reassign newcond for neuropsych conditions so algo will work *;
* All np test scores that qualified are considered             *;

data allq1;
set mitometh.filtq1;
if newcond=601 then newcond=6; 
if newcond=613 then newcond=13;
run;

x "gzip /home/mchernoff/P219C_Mito/methods/filtq1.sas7bdat";


*--------------------------------------------------*;
* 10-4-06 For first rec/patid-cond, set sortdt to 
  first of earliest seq, onsetdt, visitdt *;
* Except for cardio, hearing, dateseq is same
  for all recs in a condition by patid. For those,
  it is set for first patid record only  *;
* Recalibrate sortdt for first record of condition,
  using dateseq (earliest date for earliest abn seq,
  where relevant) *; *[Must have done this in order
  to properly assign earliest contributing event date
  for events w/ more than one contributing episode.];
*--------------------------------------------------*;
proc sort data=allq1;
by newcond patid sortdt dateseq ; *newcond = condition number*;
run;

data allq1;
set allq1;
by newcond patid;
if first.patid then sortdt=min(dateseq,sortdt); 
*format sortdt date9.; *7-11-2011 blocked because sortdt now days from birthdt*;
label sortdt="Earliest of onsetdt, header date (days since birth)"
      ;
run;


proc sort data=allq1;
by patid sortdt newcond; *newcond = condition number*;
run;

*------------------------------------------------------------*;
* reduce to one record per condition per visit date *;
*------------------------------------------------------------*;
data newmq1;
set allq1 (rename=(newcond=condq1));
by patid sortdt condq1;
if first.condq1;
run;

proc print data=newmq1 (obs=25);
title "check, newmq1, one obs per patid-sortdt-condq1, filter.";
var patid sortdt condq1 dateseq headrdt onsetdt;
run;
title;


*------------------------------------------------------------*;
* assign condition indicator *;
* output one record per patid-visit *;
*------------------------------------------------------------*;
data visdat;
set newmq1;
by patid sortdt ;
retain cond1-cond24;
array cx [*] cond1-cond24;
if first.sortdt then do;
   do i=1 to dim(cx);
    cx[i]=0;
    end;
   end;
cx[condq1]=1;
if last.sortdt then output;
keep patid sortdt dateseq headrdt onsetdt cond1-cond24;
label 
    %labmac (max= 24);
    ;
run;

proc contents data=visdat; * 7-11-11 to test macro labels *;
title "visdat";
run;

proc sort data=visdat;
by patid sortdt;
run;

proc print data=visdat (obs=25);
var patid sortdt dateseq headrdt onsetdt cond1-cond24;
title "check, visdat, one obs per patid-sortdt, filter.";
run;
title;

*------------------------------------------------------------*;
* SCORE MAJOR CONDITIONS *;
*------------------------------------------------------------*;
* identify pids w/ major condition and output all records
* with a condition that mees the criterion *;
*------------------------------------------------------------*;

data majordat;
set visdat (keep=patid sortdt cond1-cond24);
summajor=sum(of cond1-cond8, of cond14-cond21);
if summajor>=1 then major=1;
 else major=0;
if cond6 then do;
 if summajor=1 then flag6=1; * major only by condition 6 *;
  else if summajor> 1 then flag6=0; * fix 2-1-07 *;
  end;
if major=1 then output;            *select records for kids w/ major conditions only *;
keep patid sortdt cond1-cond8 cond14-cond21 summajor major flag6;
label major= "Major sign, French crit."
      flag6 = "Major sign, only cond 6, French"
      summajor="Number of major signs, French"
      ;
run;


* continue *:

proc print data=majordat (obs=25);
title "major conds, filter.";
run;

*------------------------------------------------------------*;
* SCORE MINOR CONDITIONS *;
*------------------------------------------------------------*;
* output record when first meet criterion of >= 2 minor
  conditions, each having occurred twice *;

* The current code outputs this first record meeting criterion
* and all subsequent records meeting criterion. So, for example,
* if a participant had a minor event on 3 different visits,
* a record would be output for the 2nd and 3rd visit on which
* it was reported. *;

* For each minor condition meeting criterion, the first (earliest)
* date of the two (or more reports) is also retained.
*------------------------------------------------------------*;

data minordat;
set visdat;
by patid sortdt;

retain cnt9-cnt13 cnt22-cnt24 indx9-indx13 indx22-indx24
       fdt9-fdt13 fdt22-fdt24
       flgdt9-flgdt13 flgdt22-flgdt24;

array condx [*] cond9-cond13 cond22-cond24;
array cntx [*] cnt9-cnt13 cnt22-cnt24;
array indx [*] indx9-indx13 indx22-indx24;
array fdtx [*] fdt9-fdt13 fdt22-fdt24;
array flgx [*] flgdt9-flgdt13 flgdt22-flgdt24;

if first.patid then do; * initialize counters, variables *;
  do i=1 to dim(cntx);
    cntx[i]=0;
    indx[i]=0;
    fdtx[i]=.; * 10-16-06 *; * holds first date of pair of events *;
    flgx[i]=0;               * flags when first date has been entered*;
    end;
  end;

do i=1 to dim (condx);
  if condx[i] = 1 then cntx[i]+1;                 * increment counters for minor conditions *;

  if (cntx[i] = 1 and flgx[i] = 0 ) then do;      * capture date of first of event pair *; * 10-16-06 *;
     fdtx[i]=sortdt;                              
     flgx[i]=1;
     end;

  if (cntx[i] >= 2 and indx[i]=0) then indx[i]=1; * test if there are >= 2 occurrences of a sign*;
                                                  * if so, assign indx array flag *;

  if _n_ < 3 then put /patid= sortdt=/condx[i]= cntx[i]= indx[i]=/
                                     fdtx[i]= flgx[i]=/;

  end;

sumx=sum(of indx9-indx13, of indx22-indx24) ;

if _n_ < 3 then put /patid= sumx=/;

if sumx>= 2 then do;                              * output if there are >=2 signs w/ >=2 occurrences *;
                                                  * Note: also outputs records afterwards *;

  mindate=.;                                      * 2-1-07 - Initialize mindate*;

  do i=1 to dim (condx);                          * 10-16-06 - assign earliest event date for relevant pairs *;
   if indx[i]=1 then mindate=min(fdtx[i],mindate);* Do not count if indx is not set to 1 *;

                                                  * check print *;
   if patid in (105193,105196) then 
    put patid= indx[i]= fdtx[i]= mindate=;

   end;

  output; 

  end;

if (_n_ < 100 and mindate ne .) or patid in (105193,105196)
       then put /patid= sumx= "INDX9-13,22-24: " indx9-indx13 indx22-indx24/
       mindate= "First dates 9-13/22-24: " fdt9-fdt13 fdt22-fdt24 /;

keep patid sortdt sumx indx9-indx13 indx22-indx24 mindate; * 10-16-06 added mindate *;

*format mindate fdt9-fdt13 fdt22-fdt24 date9.;  *7-11-2011;

label indx9=">= 2 occurs of cond 9"
      indx10=">= 2 occurs of cond 10"
      indx11=">= 2 occurs of cond 11"
      indx12=">= 2 occurs of cond 12"
      indx13=">= 2 occurs of cond 13"
      indx22=">= 2 occurs of cond 22"
      indx23=">= 2 occurs of cond 23"
      indx24=">= 2 occurs of cond 24"
      mindate="Earliest minor event date for qualifying pairs (days since birth)"
      sumx="#  conditions w/ qualifying pairs";
      ;
run;

proc print data=minordat (obs=25);
title "minor conds, filter.";
run;

*----------------------------------------------------------*;
* COMBINE SCORABLE CONDITIONS (THOSE WHICH MEET CRITERIA)
* FOR BOTH MAJOR AND MINOR RULES.
*----------------------------------------------------------*;
* Combine major and minor conditions records and identify
  records w/ major condition 6 only *;
*----------------------------------------------------------*;
* 10-16-06 Add minimum date for scorable minor condition 
  pairs *;
* 12-15-06 Added labels for minor, major *;
*----------------------------------------------------------*;
data dxdat;
merge majordat (in=inmajor keep=patid sortdt summajor major cond1-cond8 cond14-cond21 flag6)
      minordat (in=inminor keep=patid sortdt sumx indx9-indx13 indx22-indx24 mindate)
      ;
by patid sortdt;
if inminor then minor=1; else minor=0;
if not inmajor then do; 
  flag6=0; major=0; 
  end;
if flag6 and not inminor then newflg6=1;
 else if flag6 then newflg6=0;
 else newflg6=.;
label newflg6="Cond 6 is only major condition with no minor conditions"
      minor="Meet minor condition criterion"
      major="Meet major condition criterion";
run;

proc print data=dxdat (obs=25);
by patid;
id patid;
title "filtq1, Dxs w/ date, that meet French criterion, filter.";
run;

*----------------------------------------------------------*;
* DEVELOP SUMMARY DATASETS, WITH ONE RECORD PER PARTICIPANT ;
*----------------------------------------------------------*;
*-----------------------------------------------------------------*;
* 10-12-06 * Indicator of whether meet major / minor conditions *;
*-----------------------------------------------------------------*;

proc summary data=dxdat nway;
class patid;
var major minor;
output out=outdx max=;
run;

proc freq data=outdx;
tables major*minor/list missprint nopercent;
title "filtq1, Summary of dx conditions (ever) that meet french criterion, filter.";
run;

*---------------------------------------------------*;
* Find record corresponding to earliest visit on
* which French criterion was met *;
*---------------------------------------------------*;
* 10-16-06 * Re-assign minimum date var to account
  for cases w/ both major and minor conditions.     *;
*---------------------------------------------------*;

proc sort data=dxdat; by patid sortdt;
run;

data firstdx;
set dxdat;
by patid;
if first.patid;
fmindate=min(sortdt,mindate);       * 10-16-06 *;
if major and minor /*and (fmindate ne mindate)*/ then put  
  /"Patid has both minor/major condition on earliest record"/
   "                       " patid= major= minor=  sumx= /
   "                       " fmindate= sortdt= mindate=/
   "Major conds 1-8/14-21: " cond1-cond8 cond14-cond21/
   "Minor conds 9-13/22-24:" indx9-indx13 indx22-indx24/
  ;
*format fmindate date9.; *7-11-2011;
label fmindate="Earliest condition date (days since birth) (major or 1st event among 2 minor pairs)";
run;

proc print data=firstdx (obs=25);
id patid;
title "filtq1, First dx condition that meets French criterion, filter.";
run;

proc freq data=firstdx;
tables sumx summajor;
tables major*minor/list missprint nopercent;
title "filtq1, Summary of first dx conditions that meet french criterion, filter.";
run;

*------------------------------------------------*;
* COMPUTE AGES AT DATES CRITERIA ARE MET. ;
* NOTE ALL DATES ARE EXPRESSED AS         ;
* DAYS SINCE BIRTH                        ;
*------------------------------------------------*;
* 7-11-11 Edited code since event dates are now relative to birthdate *;

data firstdx;
merge firstdx (in=inf)
      master(in=inm keep=patid /*birthdt*/);
by patid;
*if sortdt ne . then maxage=yrdif(birthdt,sortdt,'act/act');
*if fmindate ne . then minage=yrdif(birthdt,fmindate,'act/act');
if sortdt ne . then maxage=sortdt/365.25; * 7-11-11;
if fmindate ne . then minage=fmindate/365.25; * 7-11-11;
if inm;
if not inf then do;
 major=0; minor=0;
 end;
label maxage="Age,y at mito criterion event (major or 2nd event of 2nd minor pair)"
      minage="Age,y at first event (major or 1st evt of 2 minor pairs)"
      sortdt="Mito criterion event date (major or 2nd event of 2nd minor pair)(days since birth)";
run;


*-------------------------------------------------*;
* ADD INDICATORS OF WHETHER CRITERIA WERE EVER MET*;

* For example, participant may meet major criterion
* first but subsequently also meet minor event
* criterion or vice versa.
*-------------------------------------------------*;

data firstdx;
merge firstdx
      outdx (keep=patid minor major rename=
               (minor=evminor major=evmajor));
by patid;
if evminor=. then evminor=0;
if evmajor=. then evmajor=0;
label evminor="Ever satisfy minor criterion"
      evmajor="Ever satisfy major criterion"
      ;
run;

proc means data=firstdx (where=(maxage ne .)) maxdec=3 n mean std min max ;
var maxage minage;
title "filtq1, Age met criterion";
run;

proc means data=firstdx (where=(maxage ne .)) maxdec=3 n mean std min max ;
class major minor;
var maxage minage;
title "filtq1, Age met criterion, by first type";
run;
title;

proc freq data=firstdx;
tables major*minor/list missprint nopercent;
title "filtq1, Summary of first dx conditions that meet french criterion, filter.";
run;

*-------------------------------------------------*;
* SAVE SUMMARY DATASET, ONE RECORD PER PARTICIPANT*;
* INCLUDES first condition dates, and ages, and   *;
* condition indicators for the date corresponding *;
* to the earliest meeting of the criteria         *;
*-------------------------------------------------*;

x "/bin/rm -f /home/mchernoff/P219C_Mito/methods/date_fq1.sas7bdat*";

data mitometh.date_fq1;
set firstdx;
run;

proc contents data=mitometh.date_fq1 varnum;
title "mitometh.date_fq1";
run;

x "gzip /home/mchernoff/P219C_Mito/methods/date_fq1.sas7bdat";

*------------------------------------------------------------*;
* 7-2-07 - SAVE DATASET WITH ALL SCORABLE CONDITIONS,
* INCLUDING ALL VISITS ON WHICH A MAJOR EVENT OCCURRED OR
* ON WHICH THE SECOND OR SUBSEQUENT (E.G., 3RD, ETC) MINOR
* CONDITION WAS REPORTED *;
*------------------------------------------------------------*;

x "/bin/rm -f /home/mchernoff/P219C_Mito/methods/datall_fq1.sas7bdat*";

data mitometh.datall_fq1;
set dxdat;
run;

proc contents data=mitometh.datall_fq1 varnum;
title "mitometh.datall_fq1";
run;

x "gzip /home/mchernoff/P219C_Mito/methods/datall_fq1.sas7bdat";


*-----------------*;
* STOP *;
*-----------------*;
endsas;











