/* *****************************************************************
   Directory: /home/mchernoff/P219C_Mito/sas_programs
   Pgm: find_date_q2_meth.sas
   Source Pgm: find_date_q2.sas (Oct 5, 2006)
   Date: 7-8-11
   Last edited: 7-11-11 - Edited labels and recomputed age vars
                          w/ respect to days since birth.
                7-13-11 - Additional notation.
   Purpose: 
            * MWG iDACS 616 - Case identification Query 2

            * Simplified for publication * * All patient
              identifiers have been blinded (e.g., randomly 
              assigned). All dates are expressed as
              days from birth to accomodate the de-identified
              data.

            * NOTE, BECAUSE OUR DATA DID NOT HAVE ANY SCORABLE
              METABOLIC OR MORPHOLOGIC EVENTS, THIS CODE WOULD
              HAVE TO BE REVISED TO ACCOMODATE THOSE EVENT IN
              OTHER DATASETS.

            * Find date of first mito dysfunct. determination,
              by incremental points. Scoring depends on first
              system involvement, e.g., CNS, Muscular or 
              Multisystemic. 

            * We assigned a numerical code for each of the 52
              clinical conditions, 6 of which pertain
              to the metabolic system (e.g., elevated blood lactate
              levels). Our data had no scorable metabolic events 
              (none met the MDC criteria for elevated findings).

            * SCORING ALGORITHM: The maximum score for all 
              clinical conditions is 4. Metabolic and morphologic 
              events would each add a maximum
              of 4 for a total maximum score of 12. However, in
              this example, the total maximum score is 4, using
              the reported clinical CNS, muscular and multisystemic
              events becuase our data do not include any 
              scorable metabolic or morphologic events. 
              Scoring depends on the first presenting system (CSN,
              muscular or multsystemic). Each allows a maximum
              score for events classed within the presenting system,
              and then additional points for events classed within
              the other systems, as follows.
                           
               Presenting
               System               Maximum allowable points
               ----------   -----------------------------------------------------
               Muscular:    2 points plus 1 for CNS or 2 for multisystemic events
               CNS:         2 points plus 1 for muscular or 2 for multisystemic
               Multisystem  3 points plus 1 for each muscular or multisystemic
               ----------   -----------------------------------------------------

              The scoring algorithm determines the total MDC score, the date it
              was achieved and also the dates at which incremental scores were
              reported.

                 Algorithm:
                 * reduce to first event per scorable condition
                     * separately for muscular, cns, multisystem
                       conditions (subsystems).
                     * recombine.
                 * identify first presenting system 
                 * separate records by first system 
                 * compute scores, retaining date of incremental
                   points.
                 * recombine summary records.
                 * compute ages at scores (max, and incremental).

            * Save summary dataset (one rec/patid) and also
            * Save dset w/ scorable conditions (subsystems).
                 * Can use to get constellations of 
                   conditions.
                 * score datasets have one (first) record per 
                   scorable condition (or subsystem) per date (sortdt).

            * Added two alternatives for earliest scorable
              event- one includes earliest event of np delay
              pairs (mindate), the other includes the date of the
              2nd of the pair (the scorable date, date1).

            * The input dataset has one record per condition, which
              reflects the earliest visit on which the condition was
              reported. Thus, the output dataset does not reflect 
              all the visits on which an event of a particular type
              was reported, but just the earliest.


   Input files:  mitometh.master
                 mitometh.s_condf - gr2, filtered w/indicators

   Output files: 

                 mitometh.date_fq2 - summary with one record per patid, 
                                     includes total points, dates at which 
                                     incremental point scores are met
                 mitometh.first    - first systems, new algorithm
                 mitometh.scor_fq2 - all records for scorable conditions, 
                                     one record for each (earliest report)
                                     


         Listing files: 

   Notes:  

   TASKS: 

*********************************************************************/

* call to autoexec is automatic if in same directory *;

options ls=130 ps=50;
run;


/**************************************************/
/* formats */
/**************************************************/

%include "/home/mchernoff/P219C_Mito/methods/mito_tmpformat_meth.sas";

*-------------------------------------------------*;
* macros *;
*-------------------------------------------------*;

*------------------------------------------*
* Bring in DATA    *
*------------------------------------------*;

x "gunzip  /home/mchernoff/P219C_Mito/methods/master.sas7bdat.gz";

proc sort data=mitometh.master out=master;
by patid;
run;

x "gzip  /home/mchernoff/P219C_Mito/methods/master.sas7bdat";

*---------------------------------------------------------------------------------*;
* SCORING MACRO- THIS IS APPLIED SEPARATELY TO RECORDS FOR PARTICIPANTS WITH EACH
* PRESENTING SYSTEM. E.G., IT IS APPLIED FIRST TO THOSE PARTICIPANTS PRESENTING 
* WITH A CNS EVENT. NEXT IT IS APPLIED TO THOSE PRESENTING WITH A MUSCULAR EVENT.
* FINALLY IT IS APPLIED TO THOSE PRESENTING WITH A MULTI-SYSTEMIC EVENT. THE 
* RESULTS ARE THEN RE-COMBINED TO CREATE A DATASET WITH A RECORD FOR EACH 
* PARTICIPANT, AND SUMMARY SCORES.;

* INPUT DATA CONTAINS ONE RECORD PER SCORABLE CONDITION WITHIN THE GIVEN 
* CLINICAL SYSTEM. ;

* MACRO DETERMINES TOTAL MDC SCORE AND DATES AT WHICH FINAL AND INCREMENTAL SCORES 
* ARE ACHIEVED. ;
*---------------------------------------------------------------------------------*;
*-----------------------------------------*;
* Macro to score incrementally by date *;
*-----------------------------------------*;
* assumes there may be multiple records
  per patid per visit date *;
*-----------------------------------------*;
* tot1=subtotal for muscle sys to max;
* tot2=subtotal for cns;
* tot3=subtotal for multisys;
* total=total score *;
*-----------------------------------------*;
* if come to same condition at later date,
  do not count that *;
*-----------------------------------------*;

%macro fintot (
   indset = /* input dset */,
   firstsys= /* first system */,
   m1=      /* max muscle total */,
   m2=      /* max cns total */,
   m3=      /* max multiple total */,
   outdset= /* output dset */
    );
  
data temp;
set &indset.;
run;

proc print data=temp (obs=50);
by patid;
id patid;
var patid system sortdt points muscsys cnssys multsys metasys firstsys sys1spec sys1dt;
title "&indset., &firstsys. first system, data";
run;

data temp;
set temp;
by patid sortdt;
retain tot1-tot3 predat1-predat4 date1-date4 oldtot flag1-flag4;

if first.patid then do; 
  total=0; tot23=0; tot13=0; tot12=0;
  oldtot=0;
  tot1=0; tot2=0; tot3=0; 
  predate=.;
  predat1=.; predat2=.; predat3=.; predat4=.;
  date1=.; date2=.; date3=.; date4=.;
  flag1=0; flag2=0; flag3=0; flag4=0;
  end;

* increment totals *;

if system="Muscle" then do;
    tot1=tot1+points;
    tot1=min(&m1.,tot1);  * max out at m1 *; * muscle*;   
    end;
 else if system="CNS" then do;
    tot2=tot2+points;
    tot2=min(&m2.,tot2);  * max out at m2 *; * cns *;    
    end;
 else if system="Multiple" then do;
    tot3=tot3+points;
    tot3=min(&m3.,tot3); * max out at m3 *; * multiple*;
    end;

 if _n_ < 25 or patid = 105192 then put 
     patid= system= points= tot1= tot2= tot3= oldtot=;

if last.sortdt then do; * cumulate total scores per visit *;

* max out differently depending on initial system*;
* This section adds the points for the other 2 systems to
  the points for the presenting system *;

* Note on variable naming convention: 
  tot13=tot1+tot3 - pairs of systems musc + multiple *;

/*
 tot1=min(&m1.,tot1); * max out at m1 *; * muscle*;   
 tot2=min(&m2.,tot2); * max out at m2 *; * cns *;    
 tot3=min(&m3.,tot3); * max out at m3 *; * multiple*;
 */

 %if %upcase(&firstsys.) = CNS %then %do;

  tot13=sum(tot1,tot3); * muscle/multiple*; 
  tot13=min(2,tot13);   * max out at 2*; 
  total=sum(tot2,tot13);  * final total *;

  %end;
  %else %if %upcase(&firstsys.) = MUSCLE %then %do;

   tot23=sum(tot2,tot3); * cns/multiple*;
   tot23=min(2,tot23);   * max out at 2*; 
   total=sum(tot1,tot23);  * final total *;
   
   %end;

   %else %if %upcase(&firstsys.) = MULTIPLE %then %do;

    tot12=sum(tot1,tot2); * muscle/cns *;
    tot12=min(1,tot12);   * max out at 1*; 
    total=sum(tot3,tot12);  * final total *;
   
    %end;


  total=min(4,total);     * max out at 4*;

 * Test if meet criterion for additional point *;
 * Assign  date at which point is added        *;
 * Also assign date at which score maxes out (reaches 4 points) *;

 * 10-16-06 retain first date of 2 delay conditions, if delay=1 *;
 * and contribute to a scoreable event *;

 if total>oldtot then do;
  if total>=1 and flag1=0 then do; date1=sortdt; flag1=1; 
      if delay=1 then predat1=delaydt1; 
      end;
  if total>=2 and flag2=0 then do; date2=sortdt; flag2=1;
      if delay=1 then predat2=delaydt1; 
      end;
  if total>=3 and flag3=0 then do; date3=sortdt; flag3=1; 
      if delay=1 then predat3=delaydt1; 
      end;
  if total>=4 and flag4=0 then do; date4=sortdt; flag4=1; 
      if delay=1 then predat4=delaydt1; 
      end;

  oldtot=total;
  end;

 end;

if last.patid then do;                  *output one rec per patid *;
  maxdate=max(date1,date2,date3,date4);
                                                  * 10-16-06 Earliest event among qualifying pairs of np conditions *;
                                                  * which lead to first 4 points only *;
  predate=min(predat1,predat2,predat3,predat4); 
                                                  
  onept=(date1 ne .);
  twopt=(date2 ne .);
  threept=(date3 ne .);
  fourpt=(date4 ne .);
  output;
  end;
keep patid total predate predat1-predat4 
     date1-date4 maxdate onept twopt threept fourpt
     firstsys sys1spec sys1dt;
*format predate predat1-predat4  date1-date4 maxdate date9.; * 7-11-11*; *now days relative to birth *;

label maxdate="Date of case determination (maximum score)"
      total="Total W/S Score"
      date1="Date reached 1 point(years since birth)"
      date2="Date reached 2 points(years since birth)"
      date3="Date reached 3 points(years since birth)"
      date4="Date reached 4 points(years since birth)"
      onept="Reached 1 point"
      twopt="Reached 2 points"
      threept="Reached 3 points"
      fourpt="Reached 4 points"
      predate="Earliest event among qualifying np delay event pairs(years since birth)"
      predat1="Earliest np delay leading to 1st point(years since birth)"
      predat2="Earliest np delay leading to 2nd point(years since birth)"
      predat3="Earliest np delay leading to 3rd point(years since birth)"
      predat4="Earliest np delay leading to 4th point(years since birth)"
      ;
run;

proc freq data=temp;
tables total* onept* twopt* threept* fourpt/list missprint nopercent;
title "Check on total points, patids w/ &firstsys. system first";
run;
title;


proc print data=temp (obs=50);
by patid;
id patid;
var patid firstsys sys1spec sys1dt total predat1 date1-date4 maxdate onept twopt threept fourpt;
title "&indset., &firstsys. first system, data, w/ scores";
run;

proc print data=temp (obs=20 where=(predate ne .));
by patid;
id patid;
var patid firstsys sys1spec sys1dt total predate predat1-predat4 date1-date4;
title "&indset., &firstsys. first system, data, w/ scores";
title2 "Cases w/ np delay event- shows 1st (predat[n]) and second dates (date[n])";
title3 "predate is minimum of predat1-predat4";
run;
title;

data &outdset.;
set temp;
run;
%mend;



*---------------------------------------------------------*;
* BRING IN DATA *;
*---------------------------------------------------------*;

x "gunzip  /home/mchernoff/P219C_Mito/methods/s_condf.sas7bdat.gz";


proc sort data=mitometh.s_condf out=allcond;
by patid condq2;
run;

x "gzip /home/mchernoff/P219C_Mito/methods/s_condf.sas7bdat";


proc sort data=allcond;
by patid condq2 sortdt;
run;

* select earliest event for each condition (condq2) *;

data allcond;
set allcond;
by patid condq2 sortdt;
if first.condq2;
run;

* resort by sortdt *;

proc sort data=allcond;
by patid sortdt condq2;
run;

*---------------------------------------------------------------*;
* MULTISYSTEM EVENTS *;

* WE CLASS THE MULTISYSTEM EVENTS INTO ORGAN SYSTEMS;
* WE RETAIN ONLY ONE RECORD PER EACH ORGAN SYSTEM, KEEPING TRACK
* OF CONTRIBUTING EVENT TYPES;
* 1 POINT IS ALLOWED FOR EACH ORGAN SYSTEM.;
*---------------------------------------------------------------*;
* find date of first of each multisystem condition classes *;
* Note: exacerbation of sxs, unexpected neonatal 
  death as family hx -  are not captured*;
*---------------------------------------------------------------*;
data multisys;
set allcond;
if 26 <= condq2 <=46 or condq2=2933; * 10-12-06*;
if condq2 in (26,27) then multsys=1;                      * hematology*;
 else if condq2 in (28,29,2933,30,31,32)then multsys=2;   * gi tract *; * includes f.to.thrive+short.stature*;
 else if condq2 in (33,34,35,36,37) then multsys=3;       * endocrine*;
 else if condq2 in (38,39) then multsys=4;                * heart*;
 else if condq2 in (40,41) then multsys=5;                * kidney*;
 else if condq2 in (42,43,44) then multsys=6;             * eyes*;
 else if condq2 in (45) then multsys=7;                   * ears*;
 else if condq2 in (46) then multsys=8;                   * nerve*;
if 1 <= multsys <= 8  then output;
keep patid sortdt multsys condq2;
label multsys="Multiple system class";
run;

proc sort data=multisys;
by patid sortdt multsys condq2;
run;

* reduce to one record per multiple subsystem (scorable
  condition) per visit date*;

data multisys;
length specify $25;
set multisys;
retain specify;
by patid sortdt multsys condq2;  
if first.multsys then specify="";
if specify="" then specify=trim(left(put(condq2,5.)));
 else specify=trim(left(specify))||","||trim(left(put(condq2,5.)));
specify=trim(left(specify));
if last.multsys;
run;

data multisys;
length system $10;
set multisys;
points=1;
system="Multiple";
label points="Points per condition";
run;

proc freq data=multisys; 
tables multsys*specify/list nopercent missprint;
title "Multiple systems, conditions by sortdt";
run;
title;

* 10-12-06 
  Reduce to first record per multiple subsystem (scorable
  condition);

proc print data=multisys (where=(patid= 105192));
title "Check";
run;

* REDUCE DATASET TO ONE RECORD PER ORGAN SYSTEM;

proc sort data=multisys;
by patid multsys sortdt;
run;

data multisys;
set multisys;
by patid multsys sortdt;
if first.multsys;
run;


proc freq data=multisys; 
tables multsys/list nopercent missprint;
title "First Multiple systems";
run;
title;

*---------------------------------------------------------------*;
* METABOLIC SYSTEM - OUR DATASET HAS NO SCORABLE CONDITIONS 
* IN THIS SYSTEM. IF IT DID WE WOULD HAVE TO AUGMENT CODING
* TO ACCOMODATE IT.; 

* WE ASSIGN METABOLIC SYSTEM CLASSES, ACCORDING TO MDC SCORING;
* WE SAVE ONE RECORD PER CLASS; * FURTHER PROGRAM MODIFICATIONS
* WOULD HAVE TO BE MADE TO FINTOT MACRO TO ENABLE SCORING,
* WERE THESE TYPES OF EVENTS TO BE INCLUDED IN OTHER DATASETS.;
* POINT SCORES ARE ASSIGNED HERE;
*---------------------------------------------------------------*;
* find date of first of each metabolic condition classes *;
*---------------------------------------------------------------*;
* Note -elevated CSF alanine (2 pts) unnnumbered (not captured);
*      -elevated ethylmalonic acid (1 pt) unnumb (not captured);
*---------------------------------------------------------------*;
* Note: * There are no scorable conditions- if there were,
*         then I would have to add code to select first
*         of each class *; * I added the code *[2-2-07] *;
*---------------------------------------------------------------*;
data metasys;
length specify $25 system $10;
set allcond;
if 47 <= condq2 <=52;
if condq2 in (47) then do; metasys=1; points=2; end;
 else if condq2 in (48)then do; metasys=2; points=1; end;
 else if condq2 in (49) then do; metasys=3; points=2; end;
 else if condq2 in (50) then do; metasys=4; points=2; end;
 else if condq2 in (51) then do; metasys=5; points=1; end;
 else if condq2 in (52) then do; metasys=6; points=2; end;
specify=trim(left(put(condq2,5.)));
system="Metabolic";
if 1 <= metasys <= 6  then output;
keep patid sortdt metasys condq2 points specify system;
label metasys="Metabolic system class"
      points= "Points per condition";
run;

proc print data=metasys (obs=25);
by patid;
id patid;
title "final metabolic system conditions, earliest of each with points";
run;
title;

proc freq data=metasys; 
tables specify;
title "Metabolic systems, conditions by sortdt";
run;
title;

proc freq data=metasys; 
tables metasys/list nopercent missprint;
title "All Metabolic systems";
run;
title;

* 2-2-07 * code to protect for future updates in event there were metab
  conditions * would have to select first.;

proc sort data=metasys;
by patid metasys sortdt;
run;

* REDUCE DATASET TO ONE RECORD PER METABOLIC EVENT CLASS;

data metasys;
set metasys;
by patid metasys sortdt;
if first.metasys;
run;

proc freq data=metasys; 
tables metasys/list nopercent missprint;
title "First Metabolic systems";
run;
title;

* continue *;

*-------------------------------------------------------*;
* MUSCULAR SYSTEM EVENT CLASSES ARE ASSIGNED AND ONE
* RECORD IS RETAINED FOR EACH, ALONG WITH ASSOCIATED
* POINT SCORES;
*-------------------------------------------------------*;
*-------------------------------------------------------*;
* find date of first of each muscular condition classes *;
*-------------------------------------------------------*;
data muscle;
length specify $25 system $10;
set allcond;
if 1 <= condq2 <=6;
muscsys=condq2;
points=1;
if condq2=1 then points=2;  * prog external ophthalmolplegia;
specify=trim(left(put(condq2,5.)));
system="Muscle";
if 1 <= muscsys <= 6  then output;
keep patid sortdt muscsys condq2 points specify system;
label muscsys="Muscular system class"
      points="Points per condition"
      ;
run;


proc print data=muscle (obs=25);
by patid;
id patid;
title "final muscular conditions, earliest of each with points";
run;
title;


proc freq data=muscle;
tables specify;
title "Muscle systems, conditions by sortdt";
run;
title;

proc sort data=muscle;  * 10-12-06 *;
by patid muscsys sortdt;
run;

* REDUCE DATASET TO ONE RECORD PER MUSCULAR SYSTEM EVENT CLASS;

data muscle;
set muscle;
by patid muscsys sortdt;
if first.muscsys;
run;


proc freq data=muscle;
tables specify;
title "First events, Muscle systems";
run;
title;

*--------------------------------------------------------*;
* CNS SYSTEM EVENTS ARE BROKEN DOWN INTO 3 CATEGORIES,
* THOSE RELATING TO 1) DELAYED OR ABSENT PSYCHOMOTOR
* DEVELOPMENT (DELAY IN 2 OR MORE DEVELOPMENTAL
* DOMAINS, EG., MOTOR SKILLS, LANGUAGE/SPEECH,
* COGNITION, PERSONAL/SOCIAL, ACTIVITIES OF DAILY
* LIVING, OR A REORTED DIAGNOSIS OF MENTAL RETARDATION), 
* 2) STROKE, AND 3) OTHER TYPES OF CNS  EVENTS (9 TYPES).*;

* WE HAVE INTERPRETED THE FIRST CATEGORY AS FOLLOWS:
* A DELAY IN 2 OR MORE OF THE FOLLOWING EVENT TYPES 1-6 OR
* MEETING CRITERION IN EITHER EVENT TYPE 7 OR 8.;

* 1. COMMUNICATION DISORDERS
* 2. PERVASIVE DEVELOPMENTAL DISORDERS
* 3. LEARNING DISORDERS
* 4. ATTENTION DEFICIT, DISRUPTIVE DISORDERS
* 5. COGNITIVE AND ATTENTION PROBLEMS
* 6. BEHAVIOR AND SOCIALIZATION PROBLEMS
* 7. DIAGNOSIS OF MENTAL RETARDATION
* 8. OBSERVED IMPAIRMENT IN NEUROPSCYHOLOGICAL TESTS
*    (SCORE < 70);

*----------------------------------------------------*;
* find date of first of each cns condition classes *;
*---------------------------------------------------*;

* classify cns events *;

data cns;
set allcond;
if condq2=701 then do; cnssys=1; subsys=1;end;
 else if condq2=13 then do; cnssys=1; subsys=2;  end;
 else if condq2 in (7, 8, 9, 10, 11, 12) then do; cnssys=1; subsys=3; end;
 else if condq2=14  then cnssys=2;
 else if condq2 in (15,16,17) then cnssys=3;
 else if condq2=18 then cnssys=4;
 else if condq2=19 then cnssys=5;
 else if condq2=20 then cnssys=6;
 else if condq2=21 then cnssys=7;
 else if condq2=22 then cnssys=8;
 else if condq2=23 then cnssys=9;
 else if condq2=24 then cnssys=10;
 else if condq2=25 then cnssys=11;
if 1 <= cnssys <= 11  then output;
keep patid sortdt cnssys subsys condq2;
label cnssys="CNS system class"
      subsys="Subsystem class"
      ;
run;


* MENTAL RETARDATION CRITERION *;

* find first reported mr event *;

data mr;
length specify $25;
set cns (where=(condq2=13));
by patid sortdt;
if first.patid;
mr=1;
specify="13";
keep patid sortdt cnssys subsys mr specify;
run;

* NEUROPSYCH TEST SCORE CRITERION *;

* find first reported np test event *;

data nptest;
length specify $25;
set cns (where=(condq2=701));
by patid sortdt;
if first.patid;
nptest=1;
specify="701";
keep patid sortdt cnssys subsys nptest specify;
run;

*------------------------------------------------------------*;
* DETERMINE WHETHER AT LEAST TWO CONDITIONS ARE MET IN ORDER 
* TO SATISFY THE DELAYED PSYCHOMOTOR DEVELOPMENT CRITERION.;
*------------------------------------------------------------*;
* find first date when >= 2 of the six delay conditions
  were reported (of condq2=7-12) *;
*------------------------------------------------------------*;
* these may occur on one or more visits. In addition, there
  may be > 2 conditions on these visits (e.g., 1+2=3,
  or 3 on first visit); * need to allow for that *;
* Above, we selected earliest of each type of event 
  so there cannot be a specific kind of event 
  on more than one visit. Therefore, we need to
  scan at most two visits*;
*------------------------------------------------------------*;
proc sort data=cns out=delay(where=(cnssys=1 and subsys=3));
by patid sortdt condq2;
run;

proc print data=delay;
by patid;
id patid;
var sortdt condq2;
title "check number of conds per patid";
run;

data delay;
length specdt1 specdt2 tempspec specify specrest $25 temp2 $5;
set delay;
by patid sortdt condq2;
retain delay count numcond tempspec specify
       date1 date2 numdt1 numdt2 specdt1 specdt2
       totcond critcond specrest;
if first.patid then do;
   delay=0;                            * flag for whether met minimum number of conditions *;
   count=0;                            * count for number of weeks *;
   specify="";                         * holds condition condq2 values *;
   specrest="";                        * hold conditions past criterion record *;
   date1=.; date2=.;                   * holds sort dates *;
   numdt1=0; numdt2=0;                 * numb conditions per date *;
   specdt1=""; specdt2="";             * hold conditions for each date *;
   totcond=0;                          * total conditions *;
   critcond=0;                         * criterion total  *;
  end;

if first.sortdt then do; 
   count+1;                            * increment number of visits (dates) *;
   numcond=0;                          * temp counter*;
   temp2="";                           * temp specify field*;
   tempspec="";
   end;

numcond+1;                             * count number of conditions w/in a visit *;

if count=1 then date1=sortdt;          * assign date fields.;
  else if count=2 then date2=sortdt;  

temp2=put(condq2,5.);

if numcond=1 then do;                  * augment temporary specify field*;
   tempspec=trim(left(temp2));
   end;
   else if numcond>=2 then do;
     tempspec=trim(left(tempspec))||','||trim(left(temp2));
     end;

put patid= sortdt= numcond= condq2= temp2= tempspec=;  * check *;
   
if last.sortdt then do;                * at end of each date group *;

   totcond=totcond+numcond;            * increment total number of conditions *;

   if count=1 then do;                 * assign first visit vars *;
     specdt1=tempspec;
     numdt1=numcond;
     date1=sortdt; 
     if totcond>=2 then do;            * test if criterion met *; * assign final vars *;
         critcond=totcond;             * criterion number of visits *;
         if delay =0 then delay=1;
         specify=trim(left(specdt1));
         end;
     end;

   else if count=2 and delay=0 then do; * assign 2nd visit vars, where criterion not yet met *;
     specdt2=tempspec;
     date2=sortdt; 
     numdt2=numcond;
     if totcond>=2 then do;            * test if criterion met *; * assign final vars *;
         critcond=totcond;             * criterion number of conditions for first 2 visits*;
         if delay =0 then delay=1;
         specify=trim(left(specdt1))||","||trim(left(specdt2));
         end;
     end;

     else if count >= 2 then do;       * record conditions at 2nd visit (if criter already met)
                                         /after 2nd visit date *;
      specrest=tempspec;
      end;

   if last.patid then output;          * output one record per patid *;

   if last.patid then do;
    put "                 " count=  delay= totcond= critcond= specify=/; * check *;
    end;

   end;

keep patid delay count totcond critcond specify 
     date1 date2 numdt1 numdt2 specdt1 specdt2 specrest; 
label date1="Date of 1st np delay criterion events(days since birth)"
      date2="Date of 2nd np delay criterion events(days since birth)"
      delay="Delay criterion met (2 or more of CNS condq2=7-12)"
      ;
*format date1 date2 date9.;* 7-11-11*;
run;

* select patids meeting delay criterion *;
* determine date they met criterion *;

data delay;
set delay;
sortdt=max(date1,date2);
*format sortdt date9.;* 7-11-11*;
run;

proc freq data=delay;
tables delay*totcond*numdt1*numdt2/list nopercent missprint;
tables specify*specdt1*specdt2/list nopercent missprint;
tables specify*specrest/list nopercent missprint;
title "check on delay cns system";
run;

proc print data=delay (obs=25);
title "check on delay cns system";
run;
title;

*-------------------------------------------------------------------------------*;
* COMBINE SCORABLE DELAY CONDITIONS: MENTAL RETARDATION, DELAYED PSYCHOMOTOR,
* NEUROPSYCH TEST SCORES  *;
*-------------------------------------------------------------------------------*;
* combine all scorable delay conditions- include where >= 2 pseudo domains 
  were met, where np test criterion was met, or where MR criterion was met*;
*-------------------------------------------------------------------------------*;
* determine earliest date of meeting delay criterion *;
* one rec/ patid *;* record scorable conditions*;
*-------------------------------------------------------------------------------*;
* 10-16-06 Add first date of 1st of 2 delay conditions to save *;
* 10-17-06 Only record first scorable event(s) in specify field*;
*-------------------------------------------------------------------------------*;

proc sort data=delay; by patid sortdt;
proc sort data=nptest; by patid sortdt;
proc sort data=mr; by patid sortdt;

* assign one specify field to match earliest confirmatory event *;
* this new algorithm will only pick out qualifying events that
  go w/ earliest sortdt *;

data delay;
merge delay (in=in1  where=(delay=1) 
             keep=patid sortdt delay specify specrest critcond totcond date1
             rename=(specify=delaysp date1=delaydt1))
      nptest(in=in2 keep=patid sortdt nptest specify rename=(specify=npspec))
      mr (in=in3 keep=patid sortdt specify mr rename=(specify=mrspec));
by patid sortdt;
cnsdelay=1;
cnssys=1;
label delaydt1="First date of scorable delay np condition pair (days since birth)";
run;

* REDUCE DATASET TO ONE RECORD PER DELAY CNS EVENT CLASS;

* select first scorable record per patid *;

data delay;
set delay;
by patid sortdt;
if first.patid;
run;

* document conditions which contributed to meeting the criterion *;

data delay;
length specify $25;
set delay;
if delaysp ne "" and mrspec ne "" and npspec ne "" then 
  specify=left(trim(delaysp))||","||left(trim(mrspec))||","||left(trim(npspec));
  else if mrspec ne "" and npspec ne "" then
   specify=left(trim(mrspec))||","||left(trim(npspec));
  else if delaysp ne "" and mrspec ne "" then 
   specify=left(trim(delaysp))||","||left(trim(mrspec));
  else if delaysp ne "" and npspec ne "" then 
   specify=left(trim(delaysp))||","||left(trim(npspec));
  else if npspec ne "" then  specify=left(trim(npspec));
  else if delaysp ne ""  then specify=left(trim(delaysp));
  else if mrspec ne ""  then specify=left(trim(mrspec));
specify=left(trim(specify));
run;


proc freq data=delay;
tables cnsdelay 
       specify*mr*nptest*delay*delaysp*mrspec*npspec/list nopercent missprint;
title "First cns delay condition";
run;

* For stroke conditions, consolidate records --> 
  one record per sortdt, w/ record of all conditions reported*;

data stroke;
length specify $25;
set cns (where=(cnssys=3));
by patid sortdt condq2;
retain specify;
if first.sortdt then specify="";
if specify="" then specify=trim(left(put(condq2,5.)));
 else specify=trim(left(specify))||","||trim(left(put(condq2,5.)));
specify=trim(left(specify));
if last.sortdt;
run;

proc print data=stroke (obs=15);
title 'stroke';
var patid cnssys specify condq2;
run;

* 2-2-07 * need to select first stroke event *;

proc sort data=stroke;
by patid cnssys sortdt;
run;

proc freq data=stroke;
tables cnssys;
title "ALL events, CNS stroke";
run;
title;

* REDUCE DATASET TO ONE RECORD PER STROKE CNS EVENT CLASS;

data stroke;
set stroke;
by patid cnssys sortdt;
if first.cnssys;
run;

proc freq data=stroke;
tables cnssys;
title "First events, CNS stroke";
run;
title;

* continue *;

data cnsoth;
length specify $25;
set cns (where=(cnssys=2 or cnssys>3));
specify=trim(left(put(condq2,5.)));
run;

proc print data=cnsoth (obs=15);
title 'other cns conditions';
var patid cnssys specify condq2;
run;

proc freq data=cnsoth;
tables cnssys;
title "CNS other systems";
run;

proc sort data=cnsoth;  * 10-12-06 *;
by patid cnssys sortdt;
run;

* REDUCE DATASET TO ONE RECORD EACH OF THE 9 CNS EVENT TYPES CLASSED AS -OTHER-;
* IN THIS CLASS,  THERE MAY BE MULTIPLE RECORDS PER PARTICIPANT, DEPENDING ON 
* THE NUMBER OF SCORABLE EVENTS THEY REPORTED;

data cnsoth;
set cnsoth;
by patid cnssys sortdt;
if first.cnssys;
run;

proc freq data=cnsoth;
tables cnssys;
title "First events, CNS other systems";
run;
title;

*----------------------------------------------------------------*;
* RECOMBINE ALL RECORDS FOR CNS EVENTS MEETING SCORING  CRITERIA *;
* There is one record for each scorable event;
* So, if  a participant had a significant delay, there would be
* at most one record. But if that patient also had reported    
* migraine or seizures, for example, there would be one additional
* record for each of those conditions.;
*----------------------------------------------------------------*;
* recombine all cns scorable conditions, w/ one record per each,
  earliest report date *;
* 10-16-06 Retain in dset, date of 1st of two delay criterion  and indicator*;

data newcns;
length system $10;
set cnsoth(keep=patid sortdt cnssys specify)
    delay(keep=patid sortdt cnssys specify delaydt1 delay)
    stroke(keep=patid sortdt cnssys specify)
    ;
points=1;
system="CNS";
label points= "Points per condition";
run;

proc sort data=newcns;
by patid sortdt;
run;

proc print data=newcns (obs=25);
by patid;
id patid;
title "final cns conditions, earliest of each with points";
run;
title;

proc freq data=newcns;
tables specify;
title "CNS systems, conditions by sortdt";
run;
title;

*-----------------------------------------------------------------*;
* COMBINE RECORDS FOR ALL SCORABLE CONDTIONS FROM EACH OF THE
* FOUR CLINICAL SYSTEMS. NOTE IN THIS DATASET, THERE WERE NO
* SCORABLE METABOLIC SYSTEM EVENTS SO FURTHER PROGRAM CODE
* DOES NOT CONSIDER THAT SYSTEM;
*-----------------------------------------------------------------*;
* combine all scorable conditions, one rec per condition *;
* [7-3-07 - looks to actually be earliest rec per condition]*;
* Note: specify now holds condition (condq2) numbers *;
* Note: first scorable events for each system-class. There may
* be more than one scorable event type per system-class. But, for
* each, only the record for the earliest reported is included. *;
*-----------------------------------------------------------------*;
* 10-16-06 retain first of two event dates for np delay *;

data alldata;
set   muscle (in=in3 keep=patid sortdt muscsys points system specify)
      newcns (in=in4 keep=patid sortdt cnssys points system specify delay delaydt1)
      multisys (in=in1 keep=patid sortdt multsys points system specify)
      metasys (in=in2 keep=patid sortdt metasys points system specify);
label sortdt="Event date (days since birth)"
      muscsys="Muscular system condition"
      cnssys="CNS system condition"
      metasys="Metabolic system condition"
      multsys="Multiple organ system condition"
      points="Points per system condition"
      system="System type"
      specify="Specific Conditions"
      ;
run;

proc sort data=alldata; by patid sortdt;
run;

*---------------------------------------------------*;
* COMPUTE FIRST PRESENTING CLINICAL SYSTEM *;
* IF MORE THAN ONE PRESENT ON A GIVEN DAY,
* ASSIGN FIRST PRESENTING SYSTEM AS FOLLOWS:
* MUSCULAR > CNS > MULTISYSTEM.;
*---------------------------------------------------*;
** 10-16-06 ** new code to recompute first system 
   by scorable events **;
** 10-17-06 ** try to approx original sort order w/ 
   condq2 vars.
*---------------------------------------------------*;

proc sort data=alldata (keep=patid system sortdt specify muscsys cnssys multsys
         where=(system in ("Muscle","CNS","Multiple"))) 
         out=specsys; 
by patid sortdt;
run;


* Index systems on each date *;

data specsys;
set specsys;
if system="Muscle" then muscular=1; else muscular=0;
if system="CNS" then cns=1; else cns=0;
if system="Multiple" then multi=1; else multi=0;
if muscular or cns or multi;
keep patid system specify muscular multi cns sortdt;
run;

proc summary data=specsys nway missing; 
class patid sortdt;
var muscular cns multi;
output out=xorder max=;
run;

data xorder;
set xorder;
if muscular=. then muscular=0;
if cns=. then cns=0;
if multi=. then multi=0;
run;

* Assign first system, in order: musc> cns > multi *;

data xfirst;
set xorder (where=(muscular=1 or cns=1 or multi=1));
by patid sortdt;
if first.patid;  * retain info for first date *;
keep patid firstsys sortdt;
if muscular=1 then firstsys=1;
 else if cns=1 then firstsys=2;
 else if multi=1 then firstsys=3;
* else if metab=1 then firstsys=4; 
format firstsys systemf.;
label firstsys="Earliest system involvement";
run;

* Add descriptors *;
* Record all scorable systems by date *;

proc print data=specsys (obs=25);
title "Check sort order, specsys";
var patid sortdt system specify;
run;
title;

* retain record of conds on same date *;

data descrip;
length specify2 $35;
set specsys;
retain specify2;
by patid sortdt;
if first.sortdt then specify2="";
if specify2="" then specify2=trim(left(specify));
 else specify2=trim(left(specify2))||","||trim(left(specify));
specify2=trim(left(specify2));
if _n_ < 50 then put patid= sortdt= specify= specify2=;
if last.sortdt;
keep patid sortdt specify2;
label specify2="First system conditions";
run;

proc freq data=descrip;
tables specify2;
title "Text descriptions of conditions- 1 rec per patid-date";
run;

* add desriptor *;

data xfirst;
merge xfirst (in=in1)
      descrip (rename=(specify2=specify));
by patid sortdt;
if in1;
run;

proc print data=xfirst (obs=25);
title "Earliest system involvement (of muscular, cns or multi)";
title2 "One rec-patid, 25 obs, with descrip info on first conditions, revised algo";
run;


* 7-8-2011 *;
* FINAL DATASET HOLDING EARLIEST PRESENTING SYSTEM *;

data first; 
set xfirst;
keep patid firstsys sortdt specify;
rename firstsys=newsys 
       sortdt=new1dt 
       specify=new1spec;
run;

data first;
set first;
label newsys="First system, based on scorable events/dates"
      new1dt="Date of first system determination based on scorables events(days since birth)"
      new1spec="First system conditons, revised algo, scorable events"
      ;
run;

data first;
merge first (in=inf)
      master (in=inm keep=patid);
by patid;
if inm;
if not inf then firstsys=0;  * no musc, cns or multi recorded for patid *;
run;

* continue *;

* FINAL DATASET HOLDING EARLIEST PRESENTING SYSTEM *;
*-------------------------------------------------------------------*;
* Save dataset with information on the earliest presenting clinical
  system, based on scorable systems *;
*-------------------------------------------------------------------*;
x "/bin/rm -f  /home/mchernoff/P219C_Mito/methods/first.sas7bdat*";

data mitometh.first;
set first;
run;

proc contents data=mitometh.first varnum;
title "Mitometh.first dataset";
run;

x "gzip /home/mchernoff/P219C_Mito/methods/first.sas7bdat";


* add new first system vars to alldata *;

data alldata;
merge alldata (in=in1)
      first (keep=patid newsys new1dt new1spec)
      ;
by patid;
if in1;
run;

proc freq data=alldata; * 7-8-11 *;
tables newsys new1spec/list nopercent missprint;
title "First system data (new algorithm)";
run;
title;


* clean dsets *;

proc datasets;
delete specsys xorder descrip xfirst;
run;

*--------------------------------------------------------------*;
* FINAL SCORING *;

* SPLIT FINAL DATASET CREATED ABOVE, WITH ONE RECORD PER
* SCORABLE EVENT,  DEPENDING ON THE PARTICIPANT-S
* PRESENTING CLINICAL SYSTEM, CNS, MUSCULAR OR MULTISYSTEM.    *;

* THEN APPLY SCORING MACRO (FINTOT). OUTPUT IS DATASET WITH
* ONE RECORD PER PARTICIPANT WITH THAT PRESENTING CLINICAL SYSTEM;

* FINALLY, RECOMBINE RESULTS OF SCORING MACRO FOR DATASET WITH
* ONE RECORD PER STUDY PARTICIPANT.;
*--------------------------------------------------------------*;

data muscle2 newcns2 multisys2;
set alldata (/*drop=specify*/
    rename=(new1spec=sys1spec newsys=firstsys new1dt=sys1dt)); 
if firstsys=1 then output muscle2;             * muscle system *;
 else if firstsys=2 then output newcns2;       * CNS *;
 else if firstsys=3 then output multisys2;     * Multisystem *;
run;


*------------------------------------------------------*;
* score based on first system, muscle/cns/multiple *;
* m1=muscle, m2=cns, m3=multiple- max point values *;
*------------------------------------------------------*;

%fintot (indset=muscle2, firstsys=muscle, m1=2, m2=1, m3=2, outdset=summusc2);
%fintot (indset=newcns2, firstsys=cns, m1=1, m2=2, m3=2, outdset=sumcns2);
%fintot (indset=multisys2, firstsys=multiple, m1=1, m2=1, m3=3, outdset=summult2);


* Combine summary score datasets; * 1 rec / patid *;
* A patid will only be in one of the groups *;

data sumall;
set summusc2
    sumcns2
    summult2;
run;

proc sort data=sumall;
by patid;
run;

* compute age at each point score and at max score *;
* (7-13-11, Note, dates are now expressed as days since birth) *;

* 10-16-06 2 alternative minimum dates, date1= date
  of first scorable event, mindate= date of
  first event, using 1st of 2 delay events, if 
  scorable pair occurred;

data sumall;
merge sumall (in=ins)
      master (in=inm keep=patid /*birthdt*/);
by patid;
if inm and not ins then do;
   total=0;
   firstsys=0;
   end;
if maxdate ne . then agemax=maxdate/365.25; *yrdif(birthdt,maxdate,'act/act'); * 7-11-11*;
if date1 ne . then age1=date1/365.25; *yrdif(birthdt,date1,'act/act'); * 7-11-11*;
if date2 ne . then age2=date2/365.25; *yrdif(birthdt,date2,'act/act'); * 7-11-11*;
if date3 ne . then age3=date3/365.25; *yrdif(birthdt,date3,'act/act'); * 7-11-11*;
if date4 ne . then age4=date4/365.25; *yrdif(birthdt,date4,'act/act'); * 7-11-11*;

mindate=min(predate,date1); * 10-16-06 * absolute first event date;
if mindate ne . then minage=mindate/365.25; *yrdif(birthdt,mindate,'act/act');  * 7-11-11*;

label age1="Age,y at points=1"
      age2="Age,y at points=2"
      age3="Age,y at points=3"
      age4="Age,y at points=4"
      agemax="Age,y at maximum mito score"
      mindate="(days since birth) Date of first event used in scoring (captures 1st event among np delay pairs)"
      minage="Age,y of first event used in scoring"
      ;

*format mindate date9.; * 7-11-11*;
run;

proc freq data=sumall;
tables total
       firstsys*total/list nopercent missprint;
format firstsys systemf.; * mindate date9.;* 7-11-11*;
title "Summary";
run;

proc means data=sumall maxdec=3 n mean std min max;
var agemax age1-age4;
run;

proc means data=sumall maxdec=3 n mean std min max;
class firstsys;
var agemax age1-age4;
format firstsys systemf.;
run;


proc print data=sumall (obs=20 where=(firstsys=1));
by patid;
id patid;
var /*birthdt*/ firstsys sys1spec sys1dt total agemax maxdate age1-age4;* date1-date4 ;
format agemax age1-age4 6.2;
title "s_condf--> date_fq2, Muscle first system, data, w/ scores";
run;


proc print data=sumall (obs=20 where=(firstsys=2));
by patid;
id patid;
var /*birthdt*/ firstsys sys1spec sys1dt total agemax maxdate age1-age4; * date1-date4 ;
format agemax age1-age4 6.2;
title "s_condf--> date_fq2, CNS first system, data, w/ scores";
run;

proc print data=sumall (obs=20 where=(firstsys=3));
by patid;
id patid;
var /*birthdt*/ firstsys sys1spec sys1dt total agemax maxdate age1-age4; * date1-date4 ;
format agemax age1-age4 6.2;
title "s_condf--> date_fq2, Multiple first system, data, w/ scores";
run;

*------------------------------------------------------*;
* SAVE SUMMARY DATASET WITH ONE RECORD PER PARTICIPANT *;
* SUMMARIZING MDC SCORES.;
*------------------------------------------------------*;

x "/bin/rm -f  /home/mchernoff/P219C_Mito/methods/date_fq2.sas7bdat*";

data mitometh.date_fq2;
set sumall;
run;

proc contents data=mitometh.date_fq2 varnum;
title "Mitometh.date_fq2 dataset";
run;

proc freq data=mitometh.date_fq2;
tables total;
tables total * fourpt * threept * twopt * onept/missprint list nopercent;
title "mitometh.date_fq2, MDC Scores";
run;
title;

x "gzip  /home/mchernoff/P219C_Mito/methods/date_fq2.sas7bdat";


*------------------------------------------------------*;
* SAVE SUMMARY DATASET WITH ONE RECORD PER SCORABLE
* CONDITION FOR USE IN DESCRIBING CLINICAL CHARACTERISTICS;
*------------------------------------------------------*;
*---------------------------------------------------------*;
* save dataset w/ one record per scorable condition[per 
  visit date]*;* add date of determination *;
* for work w/ constellations *;
*---------------------------------------------------------*;
* 10-16-06 Note: date1 is date of first scorable event. In
  cases where event is 2nd of 2 np delay events, mindate
  includes first of the 2 events *;
*---------------------------------------------------------*;
x "/bin/rm -f  /home/mchernoff/P219C_Mito/methods/scor_fq2.sas7bdat*";

data mitometh.scor_fq2;
merge alldata (in=ina)
      sumall (keep=patid /*birthdt*/ total agemax maxdate mindate minage date1 age1 predate);
by patid;
if ina;
format agemax 6.2;
run;

proc print data=mitometh.scor_fq2 (obs=5);
by patid sortdt;
id patid sortdt;
title "scor_fq2, 5 obs";
run;

proc contents data=mitometh.scor_fq2 varnum;
title "Mitometh.scor_fq2 dataset";
run;

x "gzip  /home/mchernoff/P219C_Mito/methods/scor_fq2.sas7bdat";

proc datasets;
delete sumall 
       summusc2 sumcns2 summult2       
       muscle2 newcns2 multisys2 
       alldata
       muscle newcns multisys metasys
       stroke cnsoth delay nptest mr
       allcond first
       check
       ;
run;


endsas;
************;
*** STOP ***;
************;

